import { Component, OnInit, DoCheck, OnDestroy, ViewChild, ViewChildren, QueryList, AfterViewChecked } from "@angular/core";
import { ActivatedRoute } from "@angular/router";

import { Observer } from "jalhyd";

import { FormulaireService } from "../../services/formulaire/formulaire.service";
import { I18nService } from "../../services/internationalisation/internationalisation.service";
import { FieldSet } from "../../formulaire/fieldset";
import { FormulaireDefinition } from "../../formulaire/definition/form-definition";
import { CalculatorResultsComponent } from "../../components/calculator-results/calculator-results.component";
import { Subscription } from "rxjs";
import { FieldSetComponent } from "../field-set/field-set.component";
import { BaseComponent } from "../base/base.component";
import { CalculatorNameComponent } from "./calc-name.component";
import { FormulaireElement } from "../../formulaire/formulaire-element";
import { FieldsetContainer } from "../../formulaire/fieldset-container";
import { FieldsetContainerComponent } from "../fieldset-container/fieldset-container.component";
import { ServiceFactory } from "../../services/service-factory";
import { MatDialog } from "@angular/material";
import { DialogConfirmCloseCalcComponent } from "../dialog-confirm-close-calc/dialog-confirm-close-calc.component";

@Component({
    selector: "hydrocalc",
    templateUrl: "./calculator.component.html",
    styleUrls: ["./calculator.component.scss"]
})
export class GenericCalculatorComponent extends BaseComponent implements OnInit, DoCheck, AfterViewChecked, OnDestroy, Observer {
    /**
     * liste des FieldSetComponent
     */
    @ViewChildren(FieldSetComponent)
    private _fieldsetComponents: QueryList<FieldSetComponent>;

    /**
     * liste des FieldsetContainerComponent
     */
    @ViewChildren(FieldsetContainerComponent)
    private _fieldsetContainerComponents: QueryList<FieldsetContainerComponent>;

    /**
     * composant d'affichage des résultats
     */
    @ViewChild(CalculatorResultsComponent)
    private resultsComponent: CalculatorResultsComponent;

    /**
     * composant "nom de la calculette"
     */
    @ViewChild(CalculatorNameComponent)
    private _calculatorNameComponent: CalculatorNameComponent;

    /**
     * formulaire affiché
     */
    private _formulaire: FormulaireDefinition;

    private _subscription: Subscription; // pour souscrire aux changements d'URL envoyés par le routeur

    /**
     * true si on a cliqué sur le bouton de lancement de calcul
     */
    private _computeClicked: boolean;

    /**
     * flag de validité gloable de la saisie
     * NB : la validité du bouton "calculer" dépend de la validité de la saisie dans l'UI et de celle indiquée par le formulaire.
     * La validité de l'UI comprend la forme (pas de chaîne alpha dans les champs numériques, etc..).
     * La validité formulaire comprend le domaine de définition des valeurs saisies.
     */
    private _isUIValid = false;

    /**
     * flag disabled du bouton "calculer"
     */
    public isCalculateDisabled = true;

    /**
     * flag (+info) indiquant un événement radio à traiter
     * nécessaire avec l'introduction du mode de valeur LINK des paramètres car quand on modifie le mode à LINK, les possibles
     * paramètres liables ne sont pas encore connus
     */
    private _pendingRadioClick = false;
    private _pendingRadioClickInfo: any;

    // services

    private intlService: I18nService;
    private formulaireService: FormulaireService;

    constructor(
        private route: ActivatedRoute,
        private confirmCloseCalcDialog: MatDialog
    ) {
        super();
        this.intlService = ServiceFactory.instance.i18nService;
        this.formulaireService = ServiceFactory.instance.formulaireService;
    }

    public get formElements(): FormulaireElement[] {
        if (this._formulaire === undefined) {
            return [];
        }
        return this._formulaire.kids as FormulaireElement[];
    }

    /**
     * détermine si un FormulaireElement est du type FieldSet
     */
    public isFieldset(fe: any): boolean {
        return fe instanceof FieldSet;
    }

    /**
     * détermine si un FormulaireElement est du type FieldsetContainer
     */
    private isFieldsetContainer(fe: any): boolean {
        return fe instanceof FieldsetContainer;
    }

    public get hasForm() {
        return this._formulaire !== undefined;
    }

    public get hasResults() {
        if (this.hasForm) {
            return this._formulaire.hasResults;
        }
        return false;
    }

    public get uitextTitre() {
        if (this.hasForm) {
            return this.formulaireService.getLocalisedTitleFromCalculatorType(this._formulaire.calculatorType);
        }
    }

    public get uitextCalculer() {
        return this.intlService.localizeText("INFO_CALCULATOR_CALCULER");
    }

    public get uitextCalculatorName() {
        return this.intlService.localizeText("INFO_CALCULATOR_CALC_NAME");
    }

    public get uitextResultsTitle() {
        return this.intlService.localizeText("INFO_CALCULATOR_RESULTS_TITLE");
    }

    ngOnInit() {
        this.intlService.addObserver(this);
        this.formulaireService.addObserver(this);
        this.formulaireService.updateLocalisation();
        this.subscribeRouter();
    }

    ngDoCheck() {
        if (this._formulaire !== undefined) {
            this.isCalculateDisabled = !(this._formulaire.isValid && this._isUIValid);
        }
    }

    ngOnDestroy() {
        this.unsubscribeRouter();
        this.formulaireService.removeObserver(this);
        this.intlService.removeObserver(this);
    }

    private subscribeRouter() {
        // récupération des paramètres passés avec la route (URL)
        this._subscription = this.route.params.subscribe(params => {
            const uid: string = params["uid"];
            this.formulaireService.setCurrentForm(uid);
        });
    }

    private unsubscribeRouter() {
        this._subscription.unsubscribe();
    }

    /*
     * gestion des événements clic sur les radios
     */
    private onRadioClick(info: any) {
        // console.log("on radio click");
        this.updateLinkedParameters();
        this._pendingRadioClick = true;
        this._pendingRadioClickInfo = info;
    }

    public ngAfterViewChecked() {
        if (this._pendingRadioClick) {
            // console.log("ng after view checked");
            this._pendingRadioClick = false;
            this._formulaire.onRadioClick(this._pendingRadioClickInfo);
            this._pendingRadioClickInfo = undefined;
        }
    }

    public onCloseForm() {
        this.formulaireService.requestCloseForm(this._formulaire.uid);
    }

    /**
     * met à jour l'interface
     */
    // private updateUI() {
    //     // this.appRef.tick();
    //     // this.changeDetectorRef.detectChanges();  // provoque une détection des changements dans les contrôles
    //     // if (!this.changeDetectorRef['destroyed']) // pour éviter l'erreur "Attempt to use a destroyed view: detectChanges"
    //     //     this.changeDetectorRef.detectChanges();
    //     this.changeDetectorRef.markForCheck();
    // }

    /**
     * relit les valeurs dans l'interface et met à jour les NgParameter
     */
    private updateParametersFromUI() {
        this._fieldsetComponents.forEach(fsc => fsc.updateParametersFromUI());
        this._fieldsetContainerComponents.forEach(fscc => fscc.updateParametersFromUI());
    }

    /**
     * met à jour les paramètres liés
     */
    private updateLinkedParameters() {
        this._fieldsetComponents.forEach(fsc => fsc.updateLinkedParameters());
        this._fieldsetContainerComponents.forEach(fscc => fscc.updateLinkedParameters());
    }

    public doCompute() {
        this.updateParametersFromUI();
        this._formulaire.doCompute();
        this._computeClicked = true;
    }

    public onCalcResultsViewChecked() {
        if (this._computeClicked) {
            this._computeClicked = false;
            this.scrollToResults();
        }
    }

    private scrollToResults() {
        const element = document.getElementById("resultsComp");
        if (element) {
            element.scrollIntoView();
        }
    }

    private getFieldsetStyleDisplay(id: string) {
        const isDisplayed: boolean = this._formulaire.isDisplayed(id);
        return isDisplayed ? "block" : "none";
    }

    private setForm(f: FormulaireDefinition) {
        if (this._formulaire !== undefined) {
            this._formulaire.removeObserver(this);
        }
        this._formulaire = f;
        if (this._formulaire !== undefined) {
            this._formulaire.addObserver(this);
        }
    }

    private updateFormulaireResults(uid: string) {
        if (this._formulaire.uid === uid) {
            this.resultsComponent.updateView();
        }
    }

    // interface Observer

    update(sender: any, data: any): void {
        if (sender instanceof I18nService) {
            this.formulaireService.updateLocalisation();
        } else if (sender instanceof FormulaireService) {
            switch (data["action"]) {
                case "currentFormChanged":
                    const uid: string = data["formId"];
                    this.setForm(this.formulaireService.getFormulaireFromId(uid));
                    this.resultsComponent.formulaire = this._formulaire;
                    this._calculatorNameComponent.model = this._formulaire;
                    break;
            }
        } else if (sender instanceof FormulaireDefinition) {
            let f: FormulaireDefinition;
            switch (data["action"]) {
                case "resetForm": // réinitialisation du formulaire
                    f = sender as FormulaireDefinition;
                    this.updateFormulaireResults(f.uid);
                    //     this.updateUI();
                    break;

                case "resultsUpdated":
                    f = sender as FormulaireDefinition;
                    this.updateFormulaireResults(f.uid);
            }
        }
    }

    /**
     * appelé après le 1er affichage du composant
     */
    protected afterFirstViewChecked() {
        this.updateUIValidity();
    }

    /**
     * calcul de la validité globale de la vue
     */
    private updateUIValidity() {
        this._isUIValid = false;

        if (this._fieldsetComponents !== undefined) {
            this._isUIValid = this._fieldsetComponents.reduce(
                // callback
                (
                    // accumulator (valeur précédente du résultat)
                    acc,
                    // currentValue (élément courant dans le tableau)
                    fieldset,
                    // currentIndex (indice courant dans le tableau)
                    currIndex,
                    // array (tableau parcouru)
                    array
                ) => {
                    return acc && fieldset.isValid;
                }
                // valeur initiale
                , this._fieldsetComponents.length > 0);
        }
        if (this._fieldsetContainerComponents !== undefined) {
            this._isUIValid = this._isUIValid && this._fieldsetContainerComponents.reduce(
                // callback
                (
                    // accumulator (valeur précédente du résultat)
                    acc,
                    // currentValue (élément courant dans le tableau)
                    fieldsetContainer,
                    // currentIndex (indice courant dans le tableau)
                    currIndex,
                    // array (tableau parcouru)
                    array
                ) => {
                    return acc && fieldsetContainer.isValid;
                }
                // valeur initiale
                , true);
        }
    }

    /**
     * réception d'un événement de validité d'un FieldSetComponent
     */
    private OnFieldsetValid() {
        this.updateUIValidity();
    }

    /**
     * réception d'un événement de validité d'un FieldsetContainerComponent
     */
    private onFieldsetContainerValid() {
        this.updateUIValidity();
    }

    /**
     * réception d'un événement de changement de valeur d'un input
     */
    private onInputChange() {
        this._formulaire.resetResults();
    }

    /**
     * flag d'affichage du bouton d'aide
     */
    public get enableHelpButton() {
        if (this._formulaire) {
            return this._formulaire.helpLink !== undefined;
        }
        return false;
    }

    private openHelp() {
        window.open("assets/docs-fr/calculators/" + this._formulaire.helpLink + "/", "_blank");
    }

    public saveCalculator() {
        this.formulaireService.saveForm(this._formulaire);
    }

    public closeCalculator() {
        const dialogRef = this.confirmCloseCalcDialog.open(
            DialogConfirmCloseCalcComponent,
            { disableClose: true }
        );
        dialogRef.afterClosed().subscribe(result => {
            if (result) {
                this.onCloseForm();
            }
        });
    }
}
